//=========================================================================
//
//	FILE:				DragDrop_DragSourceView.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			CP_View class to serve as a drag and drop CP_M_Source object.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 4 $
//						Latest Checkout:  $Date: 2005-09-09 22:34:33 -0700 (Fri, 09 Sep 2005) $
//
//=========================================================================

	// HelloWorld
#include "DragDrop_DragSourceView.h"

	// CPLAT_Framework
#include "CP_CarbonEvents.h"
#include "CP_Graphics.h"
#include "CP_Numeric.h"
#include "CP_WindowEvents.h"

const CPLAT::FlavorType kFlavorTypeColor				= CP_FOUR_CHAR_CODE('colr');

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Construction / Destruction ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	DragSourceView
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Constructor.
*	
*	@param		inSuperview			The subview handler that contains this view.
*	@param		inUpperLeft			Upper left corner of pane in releation to its superview.
*	@param		inContentSize		Content size of the view.
*	
*	@exception	none
*/
DragSourceView::DragSourceView( CPLAT::CP_View *inSuperview,
	const CPLAT::CP_Point& inUpperLeft, const CPLAT::CP_Size& inContentSize)
		: CP_View( inSuperview, inUpperLeft, inContentSize, true,
			(CPLAT::CP_E_ViewFlags)(GetFlags() | CPLAT::CP_E_ViewFlags_SupportsDragAndDrop) )
{
	InitEventHandler( GetEventHandler() );
}

//=========================================================================
//	FUNCTION:	~DragSourceView
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Destructor.
*	
*	@exception	none
*/
DragSourceView::~DragSourceView() throw()
{
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Drawing ===
#endif

//=========================================================================
//	FUNCTION:	OnDraw
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Draw the contents of this view.
*	
*	@param		inCanvas			The destination CP_Canvas object.
*	@param		inUpdateArea		The area needing to be updated.
*	
*	@exception	none
*
*	@return		void
*/
void DragSourceView::OnDraw( CPLAT::CP_Canvas& inCanvas, CPLAT::CP_Region& /*inUpdateArea*/ )
{
	CPLAT::CP_Rect			bounds = GetLocalBounds();
	CPLAT::CP_Graphics		graphics( inCanvas );
	
	graphics.StrokeRect( bounds );
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Drag Drop ===
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	AddDragImage
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Adds a drag image to the drag.
*	
*	@param		inDragDrop				The CP_DragDrop that is performing the
*										drag and drop.
*	@param		inStartPoint			The upper left corner for this image.
*	
*	@exception	none
*
*	@return		void
*/
void DragSourceView::AddDragImage( CPLAT::CP_DragDrop& inDragDrop, const CPLAT::CP_Point& inStartPoint )
{
	#if TARGET_OS_MAC
		OSStatus					status = noErr;
		RGBColor					swatchColor = { 0xFFFF, 0x0000, 0x0000 };

			// create the drag image
		GWorldPtr world;
		
		{
			GrafPtr				save;
			CPLAT::CP_Rect		bounds = GetLocalBounds();
			RGBColor	black = { 0, 0, 0 };
			
			status = NewGWorld( &world, 16, bounds, NULL, NULL, 0 );
			
			GetPort( &save );
			SetPort( world );
			
			RGBForeColor( &swatchColor );
			PaintRect( bounds );
			
			RGBForeColor( &black );
			FrameRect( bounds );
			
			SetPort( save );
		}
		
			// add the drag image to the passed in CP_DragDrop
		{
			Point			dragPoint;
			
			dragPoint.h = inStartPoint.h() - 5;
			dragPoint.v = inStartPoint.v() - 5;
			
			status = SetDragImage( inDragDrop.GetDragRef(), GetGWorldPixMap( world ), NULL, dragPoint,
					kDragStandardTranslucency );
		}
	#endif
}

//=========================================================================
//	FUNCTION:	HandleControlHitEvent
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	Handle clicks in this view to being the drag and drop process.
*	
*	@param		inEvent					The control click event when the user
*										clicks in this view.
*	
*	@exception	none
*
*	@return		void
*/
void DragSourceView::HandleControlHitEvent( CPLAT::CP_Event_Base& inEvent )
{
	CPLAT::OSStatus					status = CP_noErr;
	CPLAT::CP_RGBColor				swatchColor ( 0xFFFF, 0x0000, 0x0000 );
	CPLAT::CP_DragDrop				dragDrop;

	#if TARGET_OS_MAC
		CPLAT::CP_Event<kEventClassControl, kEventControlClick>	controlClickEvent( inEvent );
		Point					qdPoint;
		
		qdPoint.h = CPLAT::numeric_cast<SInt16>(controlClickEvent.fMouseLocation.h());
		qdPoint.v = CPLAT::numeric_cast<SInt16>(controlClickEvent.fMouseLocation.v());

		if( ::WaitMouseMoved( qdPoint ) )
		{
			AddFlavor( dragDrop, kFlavorTypeColor, &swatchColor, sizeof(swatchColor) );
			AddDragImage( dragDrop, controlClickEvent.fMouseLocation );

			TrackDrag( dragDrop, inEvent );
		}
	#endif
	
	#if TARGET_OS_WIN32
		ClearFlavors();
		
		AddFlavor( dragDrop, kFlavorTypeColor, &swatchColor, sizeof(swatchColor) );

		TrackDrag( dragDrop, inEvent );
	#endif
			
	inEvent.SetStatus( status );
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Events ===
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	InitEventHandler
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Install our event handlers. For this demo, we only install a control hit
*	handler to be notified when the user clicks in this view.
*	
*	@param		inEventHandler			The event handler to install the events to.
*	
*	@exception	none
*
*	@return		void
*/
void DragSourceView::InitEventHandler( CPLAT::CP_EventHandler& inEventHandler )
{
	#if TARGET_OS_MAC
			// install our custom view event handlers.
			// This will install event handlers on our custom (not system defined)
			// views for handling common events.
		CPLAT::CP_EventSignal			*controlHitSignal = CP_new CPLAT::CP_EventSignal();
		
		controlHitSignal->connect( slot(this, &DragSourceView::HandleControlHitEvent) );
		inEventHandler.Add( kEventClassControl, kEventControlClick, controlHitSignal );
	#endif

	#if TARGET_OS_WIN32
			// install our custom view event handlers.
			// This will install event handlers on our custom (not system defined)
			// views for handling common events.
		CPLAT::CP_EventSignal			*controlHitSignal = CP_new CPLAT::CP_EventSignal();
		
		controlHitSignal->connect( CPLAT::CP_SigSlot::slot(this, &DragSourceView::HandleControlHitEvent) );
		inEventHandler.Add( CPLAT::kCP_EventClassMouse, CPLAT::kCP_Event_MouseDown, controlHitSignal, GetViewImpBase()->GetOSControl() );
	#endif
}

#if TARGET_OS_MAC

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Mac Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_MAC

#if TARGET_OS_WIN32

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Win Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_WIN32

#if TARGET_OS_LINUX

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Linux Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_LINUX
