//=========================================================================
//
//	FILE:				CP_AutoReleasePool.h
//	
//						Created by Jens Miltner on 01.12.04.
//	
//	PURPOSE:			Support for an auto release pool.
//	
//	HISTORY:			1st Version - January 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev:$
//						Latest Checkout:  $Date:$
//
//=========================================================================
#ifndef _H_CP_AutoReleasePool
#define _H_CP_AutoReleasePool

// CPLAT_Framework
#include "CP_Types.h"
#include "CP_M_ReferenceCounted.h"
#include <vector>

CPLAT_Begin_Namespace_CPLAT

/*!
 \brief A class to manage automatic releasing of references to refounted objects.

 One common problem when using refcounted objects is that functions that create and return
 pointers to refcounted objects implicitely create a reference to the object by creating it.
 It's therefore mostly up to the caller of the function to ensure that this implicit reference
 will be released eventually.\n
 In order to move the responsibility of releasing the reference to the place where it's created,
 the auto-release pool concept comes in place: at the point of creation, the object reference can
 be added to an auto-release pool and thus is from now on managed by that auto-release pool. Eventually,
 the auto-release pool will release all the stored references.\n
 The calling function now gets an object pointer and is only responsible to create additional references
 if it wants to hold on to the object for longer than the scope of the function (actually, longer than the 
 topmost autorelease pool exists, which, in all but the most arkane situations, is at least equivalent to
 the scope of the function).\n
 Typical usage of autorelease pools is by adding the object to the topmost autorelease pool after creating it:
 \verbatim
 A* getAnA()
 {
	 A* a = new A;
	 a->AutoRelease();
	 return a;
 }
 
 void foo()
 {
	 A* a = getAnA(); // no idea whether foo creates A, but it doesn't matter, since autorelease pool takes care.
	 
	 // ... do something with a
	 
	 // no need to release a here.
 }
 \endverbatim
 \note Of course, using autorelease pool will extend the life time of the refcounted object beyond the controlled 
	   function scope. This is not by itself a problem, but one should be aware of this fact.
*/
class CP_EXPORT CP_AutoReleasePool
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
	CP_AutoReleasePool();
	//! Destructor.
	~CP_AutoReleasePool();
	//! @}
	
	//! @name Adding objects
	//! @{
	//! Add the given object reference to the topmost autorelease pool.
	static void AddObjectToTopMostPool(const CP_M_RefCounted* inObj);
	//! Add the given object reference to the this autorelease pool.
	void AddObject(const CP_M_RefCounted* inObj);
	//! @}
	
	//! Release all references inside this pool.
	void ReleaseAll();
	
	// class data
protected:
	std::vector<const CP_M_RefCounted*>	fObjects;	//!< Object references managed by this pool.
};


CPLAT_End_Namespace_CPLAT

#endif // _H_CP_AutoReleasePool

