//=========================================================================
//
//	FILE:				CP_Graphics_Imp.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Abstract base class for the implementation of CP_Graphics
//						objects that know how to draw to various OS graphics.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 232 $
//						Latest Checkout:  $Date: 2005-07-14 22:13:17 -0700 (Thu, 14 Jul 2005) $
//
//=========================================================================
#ifndef _H_CP_Graphics_Imp
#define _H_CP_Graphics_Imp
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Canvas.h"
#include "CP_ARGBColor.h"
#include "CP_Point.h"
#include "CP_Rect.h"
#include "CP_Gradient.h"
#include "CP_Graphics_Enums.h"
#include "CP_FileSystemSpec.h"
#include "CP_OSIcon.h"
#include "CP_OSPicture.h"
#include "CP_Pattern.h"
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

const SInt16 kCP_QuadCurveAccuracy		= 8;	// accuracy to draw the quad curve - higher = higher quality/slower speed

typedef long kCP_QuadCurveFixed;

typedef struct
{
	kCP_QuadCurveFixed					x;
	kCP_QuadCurveFixed					y;
} kCP_QuadCurvePoint;

typedef struct
{
	kCP_QuadCurvePoint					start;
	kCP_QuadCurvePoint					control;
	kCP_QuadCurvePoint					end;
} kCP_QuadCurveCurve;

#define kCP_QuadCurveAvg(a,b)			(((a) + (b)) / 2)
#define kCP_QuadCurveFr(x)				((x) + 0x8000 >> 16)

#ifndef TARGET_OS_LINUX
	typedef std::vector<CP_Point, std::allocator<CP_Point> >						CP_Point_Array;
	typedef std::vector<CP_Point, std::allocator<CP_Point> >::iterator				CP_Point_Iterator;
	typedef std::vector<CP_Point, std::allocator<CP_Point> >::reverse_iterator		CP_Point_ReverseIterator;
#else
	typedef std::vector<CP_Point >													CP_Point_Array;
	typedef std::vector<CP_Point >::iterator										CP_Point_Iterator;
	typedef std::vector<CP_Point >::reverse_iterator								CP_Point_ReverseIterator;
#endif

class CP_EXPORT CP_Graphics_Imp
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Destructor.
	virtual								~CP_Graphics_Imp();
	//@}

	//! @name Getters
	//@{
										/*!
										*	Returns the current width used in drawing lines.
										*	
										*	@exception	none
										*
										*	@return		float				The width of drawn lines.
										*/
	virtual	float						GetLineWidth()											{ return( fLineWidth ); }

										/*!
										*	Returns the current alpha value used when drawing.
										*	
										*	@exception	none
										*
										*	@return		float				The alpha value used in drawing.
										*/
	virtual	float						GetAlpha()												{ return( fAlpha ); }

										/*!
										*	Returns the current color used in stroking.
										*	
										*	@exception	none
										*
										*	@return		CP_ARGBColor		The color used in stroking.
										*/
	virtual	CP_ARGBColor				GetStrokeColor()										{ return( fStrokeColor ); }

										/*!
										*	Returns the current color used in filling.
										*	
										*	@exception	none
										*
										*	@return		CP_ARGBColor		The color used in filling.
										*/
	virtual	CP_ARGBColor				GetFillColor()											{ return( fFillColor ); }

										/*!
										*	Returns the CP_Canvas object to be drawn into.
										*	
										*	@exception	none
										*
										*	@return		CP_Canvas			The canvas used to draw into.
										*/
			CP_Canvas					GetCanvas()												{ return( fDestCanvas ); }
	//@}

	//! @name Setters
	//@{
										/*!
										*	Sets the alpha value used when drawing.
										*
										*	@param		inAlpha				The alpha value used in drawing.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						SetAlpha( float /*inAlpha*/ )							{}

										/*!
										*	Sets the line width used in stroking.
										*
										*	@param		inLineWidth			The line width used in drawing.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						SetLineWidth( float /*inLineWidth*/ )					{}

										/*!
										*	Sets the line join for determing how lines will be drawn when they join.
										*	Note: Some platforms do not support line join, and they will be drawn in
										*	the normal style. Current values for inLineJoint can be:
										*		- CP_E_GraphicsLineJoin_Miter
										*		- CP_E_GraphicsLineJoin_Round
										*		- CP_E_GraphicsLineJoin_JoinBevel
										*
										*	@param		inLineJoint			The way that the lines will join when drawn.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						SetLineJoin( CP_E_GraphicsLineJoin /*inLineJoin*/ )		{}

										/*!
										*	Sets the line cap for determing how lines ending will be drawn.
										*	Note: Some platforms do not support line caps, and they will be drawn in
										*	the normal style. Current values for inLineCap can be:
										*		- CP_E_GraphicsLineCap_Butt,
										*		- CP_E_GraphicsLineCap_Round,
										*		- CP_E_GraphicsLineCap_Square
										*
										*	@param		inLineCap			The way that the line ends are drawn.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						SetLineCap( CP_E_GraphicsLineCap /*inLineCap*/ )		{}

										/*!
										*	Sets the color used in stroking objects.
										*
										*	@param		inStrokeColor		The color used in stroking objects.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						SetStrokeColor( CP_ARGBColor& /*inStrokeColor*/ )		{}

										/*!
										*	Sets the color used in filling objects.
										*
										*	@param		inStrokeColor		The color used in filling objects.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
	virtual void						SetFillColor( CP_ARGBColor& /*inFillColor*/ )			{}
	//@}

	//! @name State
	//@{
										/*!
										*	Saves the current graphics state.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						SaveState()												{}

										/*!
										*	Restore the current graphics state. Each RestoreState needs to have a matching
										*	SaveState called before calling this.
										*	
										*	@code
										*		CP_Graphics				graphics( inDestCanvas );
										*
										*		graphics.SaveState();
										*
										*			do some drawing and state changes here, such as setting colors, line styles
										*
										*		graphics.RestoreState();
										*	@endcode
										*
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						RestoreState()											{}

										/*!
										*	Sets the scaling amount to be used in the current transformation matrix.
										*
										*	@param		inHorAmt			The horizontal amount to scale.
										*	@param		inVerAmt			The vertical amount to scale.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						ScaleCTM(
												float /*inHorAmt*/,
												float /*inVerAmt*/ )							{}

										/*!
										*	Sets the translation amount to be used in the current transformation matrix.
										*
										*	@param		inHorAmt				Horizontal amount to use in translation.
										*	@param		inVerAmt				Vertical amount to use in translation.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						TranslateCTM(
												float /*inHorAmt*/,
												float /*inVerAmt*/ )							{}

										/*!
										*	Sets the rotation amount to be used in the current transformation matrix.
										*
										*	@param		inAngle				The angle to use in rotation.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						RotateCTM( float /*inAngle*/ )							{}

										/*!
										*	Sets the skewing amount to be used in the current transformation matrix.
										*
										*	@param		inHorAmt			The horizontal amount to skew.
										*	@param		inVerAmt			The vertical amount to skew.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						SkewCTM(
												float /*inHorAmt*/,
												float /*inVerAmt*/ )							{}
	//@}

	//! @name Arcs
	//@{
										/*!
										*	Draws an arc using the passed in parameters, and then strokes it, using
										*	the current stroke color and stroke line width.
										*
										*	@param		inHorCenter			The horizontal center coordinate of the arc.
										*	@param		inVerCenter			The vertical center coordinate of the arc.
										*	@param		inRadius			The radius of the arc.
										*	@param		inStartAngle		The starting angle of the arc.
										*	@param		inEndAngle			The ending angle of the arc.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						StrokeArc(
												float /*inHorCenter*/,
												float /*inVerCenter*/,
												float /*inRadius*/,
												float /*inStartAngle*/,
												float /*inEndAngle*/ )							{}

										/*!
										*	Draws an arc using the passed in parameters, and then fills it, using
										*	the current fill color.
										*
										*	@param		inHorCenter			The horizontal center coordinate of the arc.
										*	@param		inVerCenter			The vertical center coordinate of the arc.
										*	@param		inRadius			The radius of the arc.
										*	@param		inStartAngle		The starting angle of the arc.
										*	@param		inEndAngle			The ending angle of the arc.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
	virtual void						FillArc(
												float /*inHorCenter*/,
												float /*inVerCenter*/,
												float /*inRadius*/,
												float /*inStartAngle*/,
												float /*inEndAngle*/ )							{}
	//@}

	//! @name Bezier Curves
	//@{
										/*!
										*	Draws a Bezier curve using the passed in parameters, and then strokes it, using
										*	the current stroke color and stroke line width.
										*
										*	@param		inStartPoint		The starting point of the curve.
										*	@param		inControlPointOne	The first control point curve.
										*	@param		inControlPointTwo	The second control point curve.
										*	@param		inEndPoint			The ending point of the curve.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						StrokeBezier(
												const CP_Point& /*inStartPoint*/,
												const CP_Point& /*inControlPointOne*/,
												const CP_Point& /*inControlPointTwo*/,
												const CP_Point& /*inEndPoint*/ )				{}

										/*!
										*	Draws a Quad Bezier curve using the passed in parameters, and then strokes it, using
										*	the current stroke color and stroke line width.
										*
										*	@param		inStartPoint		The starting point of the curve.
										*	@param		inControlPointOne	The first control point curve.
										*	@param		inEndPoint			The ending point of the curve.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						StrokeQuadBezier(
												const CP_Point& /*inStartPoint*/,
												const CP_Point& /*inControlPointOne*/,
												const CP_Point& /*inEndPoint*/ )				{}
	//@}

	//! @name Gradient
	//@{
										/*!
										*	Draws a gradient in the specified bounds. The gradient can be of the following
										*	types:
										*		- CP_E_GradientType_TopToBottom,
										*		- CP_E_GradientType_BottomToTop,
										*		- CP_E_GradientType_LeftToRight,
										*		- CP_E_GradientType_RightToLeft,
										*		- CP_E_GradientType_Radial,
										*		- CP_E_GradientType_Sunburst,
										*		- CP_E_GradientType_InsetSunburst,
										*		- CP_E_GradientType_Shapeburst,
										*		- CP_E_GradientType_Angled
										*
										*	@param		inGradientType		The type of gradient to draw. See above for
										*									acceptable types.
										*	@param		inDestBounds		The bounds of the drawn gradient.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						DrawGradient(
												CP_E_GradientType /*inGradietType*/,
												CP_Rect& /*inDestBounds*/ )						{}

										/*!
										*	Sets the angle used to draw the gradient.
										*
										*	@param		inAngle				The angle used to draw the gradient.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						SetGradientAngle( SInt16 /*inAngle*/ )					{}

										/*!
										*	Sets one of the colors used to draw the gradient.
										*
										*	@param		inIndex				Index where the color is used in drawing.
										*	@param		inColor				The color to be used.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						SetGradientColor(
												const SInt16 /*inIndex*/,
												const CP_ARGBColor& /*inColour*/ )				{}

										/*!
										*	Sets the count of the number of colors used in drawing a gradient.
										*
										*	@param		inGradientCount		The count of the number of colors to use.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						SetGradientCount( SInt16 /*inGradientCount*/ )			{}
	//@}

	//! @name Icon
	//@{
										/*!
										*	Draws a icon resource in the passed in bounds. The image will be scaled
										*	to fit the rectangle.
										*
										*	@param 		inBounds			The rectangle to draw the icon in.
										*	@param		inIconType			Type type of icon to use.
										*	@param		inIconID			Resource ID of the icon to draw.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						DrawIcon(
												const CP_Rect& /*inBounds*/,
												CP_E_IconStyle /*inIconType*/,
												SInt16 /*inIconID*/ )							{}

										/*!
										*	Draws the CP_OSicon in the passed in bounds. The image will be scaled
										*	to fit the rectangle.
										*
										*	@param		inIcon				A CP_OSIcon that contains the icon to draw.
										*	@param 		inBounds			The rectangle to draw the picture in.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						DrawIcon(
												CP_OSIcon& /*inIcon*/,
												const CP_Rect& /*inBounds*/ )					{}
	//@}

	//! @name Images
	//@{
										/*!
										*	Draws a picture image into the specified area. The image will be scaled
										*	to fit into the destination area.
										*
										*	@param		inImageResID		Picture resource id.
										*	@param		inDestBounds		The area to draw the picture into.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						DrawImage(
												SInt16 /*inImageResID*/,
												CP_Rect& /*inDestBounds*/ )						{}

										/*!
										*	Draws a picture image into the specified area. The image will be scaled
										*	to fit into the destination area. The image is located in the passed in
										*	file spec.
										*
										*	@param		inImageSpec			The location of the file containing the image
										*									to be displayed.
										*	@param		inDestBounds		The area to draw the image into.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						DrawImage(
												const CP_FileSystemSpec& /*inImageSpec*/,
												CP_Rect& /*inDestBounds*/ )						{}

										/*!
										*	Draw the passed in image object in the destination bounds,
										*	scaling the image to fit. This versions copies inSourceBounds
										*	portion of the image to the destination.
										*
										*	@param		inImageSpec			The location of the file containing the image
										*									to be displayed.
										*	@param		inSourceBounds		The source area of the image to draw.
										*	@param		inDestBounds		The area to draw the image into.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						DrawImage(
												const CP_FileSystemSpec& /*inImageSpec*/,
												CP_Rect& /*inSourceBounds*/,
												CP_Rect& /*inDestBounds*/ )						{}

										/*!
										*	Draw the passed in OSPicture in the destination bounds, scaling the image
										*	as needed to fit.
										*
										*	@param		inPicture			The picture to be drawn.
										*	@param		inBounds			The destination area to draw the picture.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						DrawPicture(
												CP_OSPicture& /*inPicture*/,
												const CP_Rect& /*inBounds*/ )					{}
	//@}

	//! @name Lines
	//@{
										/*!
										*	Draws a line from the passed in start location to the ending location.
										*
										*	@param		inStartHorLocation	The starting horizontal location.
										*	@param		inStartVerLocation	The starting vertical location.
										*	@param		inEndHorLocation	The ending horizontal location.
										*	@param		inEndVerLocation	The ending vertical location.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						Line(
												float /*inStartHorLocation*/,
												float /*inStartVerLocation*/,
												float /*inEndHorLocation*/,
												float /*inEndVerLocation*/ )					{}
	//@}

	//! @name Paths
	//@{
										/*!
										*	Marks the beginning of a path. This indicates that all subsequent drawing
										*	operations until ClosePath is called is collected into the current path.
										*	After ClosePath is called, you can then stroke or fill the path. The only
										*	drawing calls that are added, are the ones that begin with Path, such as
										*	PathMoveTo, PathAddOval, PathAddRect, etc.
										*	
										*	@code
										*		CP_Graphics				graphics( inDestCanvas );
										*
										*		graphics.BeginPath();
										*
										*			do some path drawing commands here
										*
										*		graphics.EndPath();
										*		graphics.StrokePath();
										*	@endcode
										*
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						BeginPath()												{}

										/*!
										*	Closes the current path. Every call to ClosePath needs to have a corresponding
										*	BeginPath call.
										*
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						ClosePath()												{}

										/*!
										*	Strokes the current path.
										*
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						StrokePath()											{}

										/*!
										*	Fills the current path.
										*
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						FillPath()												{}

										/*!
										*	Fills the current path with the passed in pattern.
										*
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						FillPath( CP_Pattern& /*inPattern*/ )					{}

										/*!
										*	Draws the current path using the passed in path drawing mode, which can
										*	be one of the following:
										*		- CP_E_Graphics_PathFill
										*		- CP_E_Graphics_PathEOFill
										*		- CP_E_Graphics_PathStroke
										*		- CP_E_Graphics_PathFillStroke
										*		- CP_E_Graphics_PathEOFillStroke
										*
										*	@param		inPathDrawMode		The path drawing mode to use.
										*
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						DrawPath( CP_E_PathDrawingMode /*inPathDrawMode*/ )		{}

										/*!
										*	Sets the current point in the path to that of the passed in point. This
										*	is the location where subsequent drawing operations will begin unless
										*	the call specifies a new starting point in its params.
										*
										*	@param		inPoint				The location to move to.
										*
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						PathMoveTo( const CP_Point& /*inPoint*/ )				{}

										/*!
										*	Sets the current point in the path to that of the passed in coordinates. This
										*	is the location where subsequent drawing operations will begin unless
										*	the call specifies a new starting point in its params.
										*
										*	@param		inHorLocation		The horizontal location to move to.
										*	@param		inVerLocation		The vertical location to move to.
										*
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						PathMoveTo(
												float /*inHorLocation*/,
												float /*inVerLocation*/ )						{}

										/*!
										*	Adds a line to our path. The line will be drawn from the current point to
										*	the location specified in inPoint.
										*
										*	@param		inPoint				The ending location of the line.
										*
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						PathAddLineTo( const CP_Point& /*inPoint*/ )			{}

										/*!
										*	Add inNumberPoints lines to our path. The first point is the starting point,
										*	then each additional point has a line drawn from the last point to that point.
										*
										*	@param		inPoint				An array of points.
										*	@param		inNumberPoints		The number of points in the above array of points.
										*
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						PathAddLines(
												const CP_Point /*inPoint*/[],
												SInt16 /*inNumberPoints*/ )						{}

										/*!
										*	Adds a starting and ending point to the current path.
										*
										*	@param		inStartPoint		The starting point to add.
										*	@param		inEndPoint			The ending point to add.
										*
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						PathAddLine(
												const CP_Point& /*inStartPoint*/,
												const CP_Point& /*inEndPoint*/ )				{}

										/*!
										*	Adds a line to the current path using the specified coordinates.
										*
										*	@param		inStartH			Horizontal starting point.
										*	@param		inStartV			Vertical starting point.
										*	@param		inEndH				Horizontal ending point.
										*	@param		inEndV				Vertical ending point.
										*
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						PathAddLine(
												float /*inStartH*/,
												float /*inStartV*/,
												float /*inEndH*/,
												float /*inEndV*/ )								{}

										/*!
										*	Adds a rect to the current path.
										*
										*	@param		inBounds			The bounds of the rectangle to add.
										*
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						PathAddRect( const CP_Rect& /*inBounds*/ )				{}

										/*!
										*	Adds an oval to the current path.
										*
										*	@param		inBounds			The bounds of the oval to add.
										*
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						PathAddOval( const CP_Rect& /*inBounds*/ )				{}

										/*!
										*	Adds a bezier curve to the current path.
										*
										*	@param		inStartPoint		The starting point of the curve.
										*	@param		inControlPointOne	The first control point curve.
										*	@param		inControlPointTwo	The second control point curve.
										*	@param		inEndPoint			The ending point of the curve.
										*
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						PathAddBezier(
												const CP_Point& /*inStartPoint*/,
												const CP_Point& /*inControlPointOne*/,
												const CP_Point& /*inControlPointTwo*/,
												const CP_Point& /*inEndPoint*/ )				{}

										/*!
										*	Adds a quadratic bezier curve to the current path.
										*
										*	@param		inStartPoint		The starting point of the curve.
										*	@param		inControlPointOne	The first control point curve.
										*	@param		inEndPoint			The ending point of the curve.
										*
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						PathAddQuadBezier(
												const CP_Point& /*inStartPoint*/,
												const CP_Point& /*inControlPointOne*/,
												const CP_Point& /*inEndPoint*/ )				{}

										/*!
										*	Adds a polygon to the current path as specified in the passed in points array.
										*
										*	@param		inPoints			An array of points to draw the polygon.
										*
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						PathAddPoly( CP_Point_Array& /*inPoints*/ )				{}

										/*!
										*	Add an arc of a circle to the context's path, possibly preceded by a straight
										*	line segment.  `(inHorCenter, inVerCenter)' is the center of the arc; `inRadius'
										*	is its radius; `inStartAngle' is the angle to the first endpoint of the arc;
										*	`inEndAngle' is the angle to the second endpoint of the arc; and `inClockwise'
										*	is 1 if the arc is to be drawn clockwise, 0 otherwise. `inStartAngle' and `inEndAngle'
										*	are measured in radians.
										*
										*	@param		inHorCenter			The horizontal center coordinate of the arc.
										*	@param		inVerCenter			The vertical center coordinate of the arc.
										*	@param		inRadius			The radius of the arc.
										*	@param		inStartAngle		The starting angle of the arc.
										*	@param		inEndAngle			The ending angle of the arc.
										*	@param		inClockwise			Flag to indicate if the arc is drawn clockwise or
										*									counter clockwise.
										*
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						PathAddArc(
												float /*x*/,
												float /*y*/,
												float /*radius*/,
												float /*startAngle*/,
												float /*endAngle*/,
												SInt16 /*clockwise*/ )							{}

										/*!
										*	Sets the clipping path of the canvas being drawn into to that of the current
										*	path.
										*
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						ClipPath()												{}
	//@}

	//! @name Polygons
	//@{
										/*!
										*	Draws the polygon specified by the passed in points array, then strokes it.
										*
										*	@param		inPoints			Points of the polygon to be drawn.
										*
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						StrokePoly( CP_Point_Array& /*inPoints*/ )				{}

										/*!
										*	Draws the polygon specified by the passed in points array, and fills it.
										*
										*	@param		inPoints			Points of the polygon to be drawn.
										*
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						FillPoly( CP_Point_Array& /*inPoints*/ )				{}
	//@}

	//! @name Rects
	//@{
										/*!
										*	Draws a rect specified by the passed in bounds, then strokes it.
										*
										*	@param		inBounds			Area of the rect to be drawn.
										*
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						StrokeRect( const CP_Rect& /*inBounds*/ )				{}

										/*!
										*	Erases the passed in rect  using the canvases current background color.
										*
										*	@param 		inBounds			The rectangle to erase.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						EraseRect( const CP_Rect& /*inBounds*/ )				{}

										/*!
										*	Draws a rect specified by the passed in bounds, then fills it.
										*
										*	@param		inBounds			Area of the rect to be drawn.
										*
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						FillRect( const CP_Rect& /*inBounds*/ )					{}
	//@}
			
	//! @name Regions
	//@{
										/*!
										*	Draws a region specified by the passed in region, then strokes it.
										*
										*	@param		inRegion			Region to be drawn.
										*
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						StrokeRegion( const CP_Region& /*inRegion*/ )			{}

										/*!
										*	Draws a region specified by the passed in region, then fills it.
										*
										*	@param		inRegion			Region to be drawn.
										*
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						FillRegion( const CP_Region& /*inRegion*/ )				{}
	//@}

	//! @name Round Rects
	//@{
										/*!
										*	Draws a rounded rect specified by the passed in rect, then strokes it.
										*
										*	@param		inBounds			The bounds of the rect to be drawn.
										*	@param		inHorRadius			Horizontal radius of the corners.
										*	@param		inVerRadius			Vertical radius of the corners.
										*
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						StrokeRoundRect(
												const CP_Rect& /*inBounds*/,
												SInt16 /*inHorRadius*/,
												SInt16 /*inVerRadius*/ )						{}

										/*!
										*	Draws a rounded rect specified by the passed in rect, then fills it.
										*
										*	@param		inBounds			The bounds of the rect to be drawn.
										*	@param		inHorRadius			Horizontal radius of the corners.
										*	@param		inVerRadius			Vertical radius of the corners.
										*
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						FillRoundRect( 
												const CP_Rect& /*inBounds*/,
												SInt16 /*inHorRadius*/,
												SInt16 /*inVerRadius*/ )						{}
	//@}

	//! @name Oval / Circle
	//@{
										/*!
										*	Draws an oval specified by the passed in rect, then strokes it.
										*
										*	@param		inBounds			The bounds of the oval to be drawn.
										*
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						StrokeOval( const CP_Rect& /*inBounds*/ )				{}

										/*!
										*	Draws an oval specified by the passed in rect, then fills it.
										*
										*	@param		inBounds			The bounds of the oval to be drawn.
										*
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						FillOval( const CP_Rect& /*inBounds*/ )					{}
	//@}

	//! @name Text
	//@{
										/*!
										*	Sets a flag to indicate how the text will be drawn in the destination canvas,
										*	if the platform supports them. Such as transparent or opaque.
										*
										*	@param		inFlags				The flags to set.
										*
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						SetTextRenderingFlags( SInt16 /*inFlags*/ )				{}

										/*!
										*	Draws the passed in text at the locatio specified. The text will not be
										*	wrapped.
										*
										*	@param		inFontSpec			FontSpec object that describes the font and
										*									justification of the text.
										*	@param		inText				The text to be drawn.
										*	@param		inStartPoint		The starting point to draw the text.
										*
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						DrawString(
												CP_FontSpec& /*inFontSpec*/,
												const CP_String& /*inText*/,
												const CP_Point& /*inStartPoint*/ )				{}

										/*!
										*	Draws the passed in text at the location specified. The text will be
										*	wrapped to fit in the passed in bounds.
										*
										*	@param		inFontSpec			FontSpec object that describes the font and
										*									justification of the text.
										*	@param		inText				The text to be drawn.
										*	@param		inBounds			The bounds to draw the text in.
										*	@param		inHorAlign			The horizontal alignment of the text. It should
										*									be one of the following:
										*										- CP_E_TextAlign_HorLeft
										*										- CP_E_TextAlign_HorCenter
										*										- CP_E_TextAlign_HorRight
										*	@param		inVerAlign			The vertical alignment of the text. It should
										*									be one of the following:
										*										- CP_E_TextAlign_VerTop
										*										- CP_E_TextAlign_VerMiddle
										*										- CP_E_TextAlign_VerBottom
										*	@param		inWrapText			If true, then the text is wrapped at the right
										*									edge of the passed in bounds.
										*
										*
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						DrawText(
												CP_FontSpec& /*inFontSpec*/,
												const CP_String& /*inText*/,
												const CP_Rect& /*inBounds*/,
												SInt16 /*inHorAlign = CP_E_TextAlign_HorLeft*/,
												SInt16 /*inVerAlign = CP_E_TextAlign_VerTop*/,
												bool /*inWrapText = true*/ )					{}

										/*!
										*	Draws the passed in text at the locatio specified. The text will be
										*	truncated to fit in the passed in bounds.
										*
										*	@param		inFontSpec			FontSpec object that describes the font and
										*									justification of the text.
										*	@param		inText				The text to be drawn.
										*	@param		inBounds			The bounds to draw the text in.
										*	@param		inHorAlign			The horizontal alignment of the text. It should
										*									be one of the following:
										*										- CP_E_TextAlign_HorLeft
										*										- CP_E_TextAlign_HorCenter
										*										- CP_E_TextAlign_HorRight
										*	@param		inVerAlign			The vertical alignment of the text. It should
										*									be one of the following:
										*										- CP_E_TextAlign_VerTop
										*										- CP_E_TextAlign_VerMiddle
										*										- CP_E_TextAlign_VerBottom
										*
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						DrawTrunc(
												CP_FontSpec& /*inFontSpec*/,
												const CP_String& /*inText*/,
												const CP_Rect& /*inBounds*/,
												SInt16 /*inHorAlign = CP_E_TextAlign_HorLeft*/,
												SInt16 /*inVerAlign = CP_E_TextAlign_VerTop*/ )	{}
	//@}

protected:
	//! @name Construction / Destruction
	//@{
										CP_Graphics_Imp( CP_Canvas& inDestCanvas );
	//@}

	// class data
protected:
	CP_Canvas							fDestCanvas;			/*! Destination canvas to draw into. */

	float								fAlpha;					/*! Alpha value for drawing. */
	float								fLineWidth;				/*! Line width for the current stroking. */
	CP_ARGBColor						fStrokeColor;			/*! Color used in stroking. */
	CP_ARGBColor						fFillColor;				/*! Color used in filling. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Graphics_Imp
