//=========================================================================
//
//	FILE:				CP_MemoryRecording.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Classes used in recording various CPLAT memory allocations
//						and deallocations for use in leak detection.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 225 $
//						Latest Checkout:  $Date: 2005-07-01 17:21:21 -0700 (Fri, 01 Jul 2005) $
//
//=========================================================================
#ifndef _H_CP_MemoryRecording
#define _H_CP_MemoryRecording
#if ! TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_FileSystemSpec.h"
#include "CP_File.h"
#include "CP_Heap.h"
#include "CP_Types.h"

	// Standard Library
#include <stdio.h>
#include <vector>

CPLAT_Begin_Namespace_CPLAT

	// memory block types we are recording
typedef enum E_CP_BlockType
{
	BT_Handle,
	BT_Ptr,
	BT_PtrToHand,
	BT_HandToHand,
	
	BT_Resource,
	
	BT_ColorTable,
	BT_GWorld,
	BT_Menu,
	BT_MenuBar,
	BT_Picture,
	BT_PixMap,
	BT_Region,
	BT_Window,
	BT_AppleEvent,
	BT_CoreFoundation,
	
	BT_Win_Bitmap,			// Win32 objects
	BT_Win_Brush,
	BT_Win_DC,
	BT_Win_Font,
	BT_Win_GDIShape,
	BT_Win_Palette,
	BT_Win_Pen,
	BT_Win_Memory
} E_CP_BlockType;

enum
{
	CP_BS_NonLeak = 0x00000001,
	CP_BS_Deleted = 0x00000010,
	CP_BS_Allocated = 0x00000100
};

typedef UInt32	CP_BlockMarker;

	// maximum length of a sessions name
const SInt16	kCP_MaxSessionNameLength							= 25;

//=======================================================================
// CP_MemoryAllocation * - this class is responsible for information about
//			one memory block allocation.
//=======================================================================
class CP_EXPORT CP_MemoryAllocation
{
public:
	SInt16			fBlockType;			// what type of memory block are we looking at
	SInt32			fAddress;			// address of block
	SInt16			fBlockSize;			// size of block
	SInt16			fSession;			// session that this block was allocated in
	const char		*fFunction;			// function memory was allocated in
	const char		*fFile;				// file memory was allocated in
	SInt16			fLine;				// line memory was allocated on in fFile
	SInt32			fBlockStatus;		// state this memory block is in

public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_MemoryAllocation();
										CP_MemoryAllocation( const CP_MemoryAllocation& inAllocation );
	//@}

	//! @name Getters / Setters
	//@{
										/*!
										*	Returns the size of the allocated block.
										*	
										*	@exception	none
										*	
										*	@return		SInt32		The allocated blocks size.
										*/
			SInt32						GetBlockSize()									{ return( fBlockSize ); }
			
										/*!
										*	Sets the current status of this block.
										*
										*	@param		inBlockStatus	The status of the block.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetBlockStatus( SInt32 inBlockStatus )			{ fBlockStatus = inBlockStatus; }

										/*!
										*	Returns the current status of this block.
										*	
										*	@exception	none
										*	
										*	@return		SInt32		The current status of the block.
										*/
			SInt32						GetBlockStatus()								{ return( fBlockStatus ); }
	//@}
			
	//! @name operators
	//@{
			CP_MemoryAllocation&		operator =( const CP_MemoryAllocation& inAllocation );
			bool						operator ==( const CP_MemoryAllocation& inAllocation );
	//@}
};

#if ! TARGET_OS_LINUX
	typedef std::vector<CP_MemoryAllocation, std::allocator<CP_MemoryAllocation> >						CP_Resource_List;
	typedef std::vector<CP_MemoryAllocation, std::allocator<CP_MemoryAllocation> >::iterator			CP_Resource_Iterator;
	typedef std::vector<CP_MemoryAllocation, std::allocator<CP_MemoryAllocation> >::reverse_iterator	CP_Resource_ReverseIterator;
#else
	typedef std::vector<CP_MemoryAllocation>															CP_Resource_List;
	typedef std::vector<CP_MemoryAllocation>::iterator													CP_Resource_Iterator;
	typedef std::vector<CP_MemoryAllocation>::reverse_iterator											CP_Resource_ReverseIterator;
#endif

//=======================================================================
// CP_MemorySession - this class is responsible for maintaining information
//			our current memory tracking session.
//=======================================================================
class CP_EXPORT CP_MemorySession
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_MemorySession( char *inSessionName );
	//! Destructor.
	virtual								~CP_MemorySession();
	//@}

	//! @name Getters
	//@{
										/*!
										*	Returns the name of this session.
										*	
										*	@exception	none
										*	
										*	@return		char *		The name of the session.
										*/
			char						*GetName()										{ return( &fSessionName[0] ); }
	//@}
			
	//! @name operators
	//@{
	static	void						*operator new( size_t size );
	static	void						operator delete( void *ptr );
	//@}

	// class data
protected:
	char								fSessionName[ 256 ];	// name of current tracking session	

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

#if ! TARGET_OS_LINUX
	typedef std::vector<CP_MemorySession *, std::allocator<CP_MemorySession *> >					CP_Session_List;
	typedef std::vector<CP_MemorySession *, std::allocator<CP_MemorySession *> >::iterator			CP_Session_Iterator;
	typedef std::vector<CP_MemorySession *, std::allocator<CP_MemorySession *> >::reverse_iterator	CP_Session_ReverseIterator;
#else
	typedef std::vector<CP_MemorySession *>															CP_Session_List;
	typedef std::vector<CP_MemorySession *>::iterator												CP_Session_Iterator;
	typedef std::vector<CP_MemorySession *>::reverse_iterator										CP_Session_ReverseIterator;
#endif

//=======================================================================
// CP_MemoryTracker - this class is responsible for maintaining a list of
//			MemoryTrackSessions that our currently recording memory
//			allocations / deallocations.
//=======================================================================
class CP_EXPORT CP_MemoryTracker
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_MemoryTracker();
	//! Destructor.
	virtual								~CP_MemoryTracker();
	//@}
	
	//! @name Instance
	//@{
	static	CP_MemoryTracker			*Instance();
	//@}

	//! @name Recording
	//@{
										/*!
										*	Turns memory recording on.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						StartRecording()								{ fAllowRecording = true; }

										/*!
										*	Turns memory recording off.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						StopRecording()									{ fAllowRecording = false; }

										/*!
										*	Returns true if memory recording is currently turned on.
										*	
										*	@exception	none
										*	
										*	@return		bool		True if memory recording is currently on.
										*/
			bool						IsRecording()									{ return( fAllowRecording ); }
			
			void						RecordAllocation(
												E_CP_BlockType inBlockType,
												void *inMemoryBlocksAddress,
												SInt32 inBlockSize,
												const char *inFunction,
												const char *inFile,
												SInt16 inLine );
			void						RecordDeallocation(
												void *inMemoryBlocksAddress,
												bool inDoubleDeleteDetection = false );

			void						DumpTrackingResults();
			void						DumpCurrentSessionTrackingResults( char *inFileName );
			void						DumpSessionTrackingResults(
												char *inFileName,
												SInt16 inSession );
			void						DumpSessionTrackingResults(
												const CP_FileSystemSpec& inFileSpec,
												SInt16 inSession );
										
			void						MarkAllBlocksStatus( SInt32 inBlockStatus );
			void						MarkAllBlocksNonLeaks();
	//@}

	//! @name operators
	//@{
	static	void						*operator new( size_t size );
	static	void						operator delete( void *ptr );
	//@}

	//! @name Sessions
	//@{
			void						BeginSession( char *inSessionName );
			void						EndSession();
			void						EndSession( const CP_FileSystemSpec& inFileSpec );

										/*!
										*	Returns the index of the current session in the list of
										*	sessions.
										*	
										*	@exception	none
										*	
										*	@return		SInt32		The current session index.
										*/
			SInt32						GetCurrentSession()								{ return( fCurrentSession ); }
			CP_String					GetCurrentSessionName() const;

										/*!
										*	Returns the list of sessions.
										*	
										*	@exception	none
										*	
										*	@return		CP_Session_List		List of memory recording sessions.
										*/
			CP_Session_List				*GetSessionsList()								{ return( &fSessionsList ); }
									
										/*!
										*	Returns the size of the debug stack recorded with each allocation.
										*	
										*	@exception	none
										*	
										*	@return		SInt16		Size of the debug stack.
										*/
			SInt16						GetDebugStackCrawlDepth()						{ return( fDebugStackCrawlDepth ); }

										/*!
										*	Sets the size of the debug stack recorded with each allocation.
										*	
										*	@param		inDepth		Size of the debug stack to use with each
										*							memory recording.
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetDebugStackCrawlDepth( SInt16 inDepth )		{ fDebugStackCrawlDepth = inDepth; }
	//@}

private:
	//! @name Recording
	//@{
			void						DumpAllAllocations( CP_File& outFile );
			SInt32						CountLeaks();
	//@}

	//! @name Sessions
	//@{
			void						DumpSessionAllocations(
												CP_File& outFile,
												SInt32 inSessionIndex );
			UInt32						CountSessionAllocations( SInt32 inSessionIndex );
	//@}

protected:
	static CP_MemoryTracker				*gInstance;				/*!< Pointer to ourselves. */

	bool								fAllowRecording;		/*!< Used to temporarlly turn recording off. */

	CP_Resource_List					fAllocatedResources;	/*!< List of allocated resources. */
	
	CP_Session_List						fSessionsList;			/*!< List of sessions. */
	SInt32								fCurrentSession;		/*!< Current recording session. */
	
	SInt16								fDebugStackCrawlDepth;	/*!< Length of our stack crawl in memory recordings. */

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_MemoryRecording


