//=========================================================================
//
//	FILE:				CP_OutStream.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Classes that provide streaming support for writing data
//						into streams. There are classes that will stream the
//						data to a memory block or file.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 106 $
//						Latest Checkout:  $Date: 2005-03-21 15:03:06 -0800 (Mon, 21 Mar 2005) $
//
//=========================================================================
#ifndef _H_CP_OutStream
#define _H_CP_OutStream
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_FileSystemSpec.h"
#include "CP_Point.h"
#include "CP_Rect.h"
#include "CP_RGBColor.h"
#include "CP_StreamDefines.h"
#include "CP_String.h"

	// MSL
#include <map>

CPLAT_Begin_Namespace_CPLAT

	// forward class declarations
class CP_EXPORT CP_M_Reanimatable;
class CP_EXPORT CP_HandleBlock;
class CP_EXPORT CP_PtrBlock;

/*!
*	@class		CP_Outstream
*	@brief		Abstract streaming class for writing data to a stream.
*/
class CP_EXPORT CP_OutStream
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_OutStream(
												bool inHasHeader,
												bool inHasDebugTags = false );
										CP_OutStream( const CP_OutStream& inStream );
	//! Destructor.
	virtual								~CP_OutStream() throw();
	//@}

	//! @name Writing
	//@{
	virtual void 						WriteBytes(
												const void *inBuffer,
												UInt32 inNumberBytes );
	//@}

	//! @name ByteSwapping
	//@{
			bool 						NeedsByteSwap() const						{ return( fNeedsByteSwap ); }
			void						SetNeedsByteSwap( bool doSwap )				{ fNeedsByteSwap = doSwap; }
	//@}
	
	//! @name Getters
	//@{
										/*!
										*	Returns the stream version used to write this data.
										*	
										*	@exception	none
										*	
										*	@return		UInt16			The version of this stream data.
										*/
			UInt16						GetStreamVersion()	const					{ return( fCPLATStreamVersion ); }
			
			bool						HasHeader() const							{ return( fHasHeader ); }
	//@}

	//! @name Positioning
	//@{
										/*!
										*	Return the current position of our marker. Subclasses must override this.
										*	
										*	@exception	none
										*	
										*	@return		UInt32			The current position of the stream marker.
										*/
	virtual UInt32						GetPosition() const							{ return( 0 ); }

										/*!
										*	Sets the current position of our marker. Subclasses must override this.
										*
										*	@param		inPosition		The position to set the stream marker to.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
	virtual void						SetPosition( UInt32 /*inPosition*/ )		{}
	//@}

	//! @name Operators
	//@{
			CP_OutStream&				operator = ( const CP_OutStream& inStream );
	//@}

	//! @name Tags
	//@{
										/*!
										*	Returns a flag to indicate if the data in the stream is tagged with the
										*	type of data written before the actual data. Used in debugging streams.
										*	
										*	@exception	none
										*	
										*	@return		bool const			Returns true if the data is tagged.
										*/
			bool 						HasTaggedData() const						{ return( fHasTaggedData ); }
			void 						WriteTag( SInt32 inTag );
	//@}

protected:
	//! @name Writing
	//@{
										/*!
										*	Writes out bytes to the stream. Subclasses must override to handle writing
										*	the data to the output stream.
										*
										*	@param		inDestBuffer		The buffer to hold the data.
										*	@param		inNumberBytes		The number of bytes to write.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
	virtual void 						OnWriteBytes(
												const void */*inDestBuffer*/,
												UInt32 /*inNumberBytes*/ )			{}
	
	virtual void 						WriteHeader();
	//@}

protected:
	bool								fHeaderWasWritten;		/*! Flag to indicate if the header has been written to the stream. */
	bool								fNeedsByteSwap;			/*! Flag to indicate if we need to byte swap the data. */
	UInt32								fCurrentPosition;		/*! The current position in the stream to write the next data to. */
	UInt16								fCPLATStreamVersion;	/*! Version of the stream data. */
	bool								fHasTaggedData;
	bool								fHasHeader;
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif
};

CP_OutStream& operator << ( CP_OutStream& stream, const bool& inBool );		
CP_OutStream& operator << ( CP_OutStream& stream, const char& inchar );				
CP_OutStream& operator << ( CP_OutStream& stream, const char *inchar );
CP_OutStream& operator << ( CP_OutStream& stream, const unsigned char& inUchar );
CP_OutStream& operator << ( CP_OutStream& stream, const unsigned char *inchar );
CP_OutStream& operator << ( CP_OutStream& stream, const SInt16& inSInt16 );
CP_OutStream& operator << ( CP_OutStream& stream, const UInt16& inUInt16 );
CP_OutStream& operator << ( CP_OutStream& stream, const SInt32& inSInt32 );
CP_OutStream& operator << ( CP_OutStream& stream, const UInt32& inUInt32 );
CP_OutStream& operator << ( CP_OutStream& stream, const float& inFloat );
CP_OutStream& operator << ( CP_OutStream& stream, const double& inDouble );
CP_OutStream& operator << ( CP_OutStream& stream, const CP_String& inString );
CP_OutStream& operator << ( CP_OutStream& stream, const CP_Point& inPoint );
CP_OutStream& operator << ( CP_OutStream& stream, const CP_Size& inSize );
CP_OutStream& operator << ( CP_OutStream& stream, const CP_Rect& inRect);
CP_OutStream& operator << ( CP_OutStream& stream, const CP_RGBColor& inRGBColor );
CP_OutStream& operator << ( CP_OutStream& stream, const CP_HandleBlock& inHandle );
CP_OutStream& operator << ( CP_OutStream& stream, const CP_PtrBlock& inPtr );
CP_OutStream& operator << ( CP_OutStream& stream, const CP_FileSystemSpec& inFileSpec );

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_OutStream

